#include <QOpenGLWidget>
#include <QElapsedTimer>
#include <QApplication>
#include <QPushButton>
#include <QGLWidget>
#include <QLayout>
#include <QTimer>
#include <QLabel>

class Widget : public QWidget
{
public:
	Widget() :
		QWidget(),
		layout( this ),
		addOpenGL( "Add QOpenGLWidget" ),
		addGL( "Add QGLWidget" ),
		label( "? FPS" ),
		counter( 0 )
	{
		label.setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
		layout.setMargin( 3 );
		layout.addWidget( &label );
		layout.addWidget( &addOpenGL );
		layout.addWidget( &addGL );
		connect( &addOpenGL, &QPushButton::clicked, [ this ]
		{
			layout.addWidget( &openGLW ); //add QOpenGLWidget
			removeButtons();
		} );
		connect( &addGL, &QPushButton::clicked, [ this ]
		{
			layout.addWidget( &glW ); //add QGLWidget
			removeButtons();
		} );
		connect( &timer, &QTimer::timeout, [ this ]
		{
			update(); //update the raster window (it calls paintEvent())
			++counter; //increment the FPS counter
			if ( elapsedTimerSec.elapsed() >= 500 ) //if 0.5 second elapsed...
			{
				label.setText( QString( "%1 FPS" ).arg( counter * 1000000000.0 / elapsedTimerSec.nsecsElapsed(), 0, 'f', 2 ) ); //show the FPS
				//reset counters
				counter = 0;
				elapsedTimer.restart();
				elapsedTimerSec.restart();
			}
		} );
		//start counters
		elapsedTimer.start();
		elapsedTimerSec.start();
		timer.start( 0 ); //run at max speed
	}
private:
	void removeButtons()
	{
		layout.removeWidget( &addOpenGL );
		layout.removeWidget( &addGL );
		addOpenGL.close();
		addGL.close();
	}

	QOpenGLWidget openGLW;
	QGLWidget glW;

	QVBoxLayout layout;
	QPushButton addOpenGL, addGL;
	QLabel label;

	QElapsedTimer elapsedTimer, elapsedTimerSec;
	QTimer timer;

	quint32 counter;
};

int main( int argc, char *argv[] )
{
//	setenv( "vblank_mode", "0", true );

	QApplication app( argc, argv );

	Widget w;
	w.resize( 800, 480 );
	w.show();

	return app.exec();
}
